/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution;

public interface DistributableResource {

	// resource types
	public static enum RESOURCE_TYPE {
		ENERGY, SHELTER
	}

	/**
	 * @return true if the resource is empty, false otherwise
	 */
	public boolean isEmpty();

	/**
	 * Get this resource's type
	 * 
	 * @return
	 */
	public RESOURCE_TYPE getResourceType();


	/**
	 * Set amount if this resource
	 * 
	 * @param amount
	 */
	public void setAmount(double amount);

	public void setPercentagedAmount(double percentage);

	/**
	 * Get amount of this resource
	 * 
	 * @return
	 */
	public double getAmount();

	public double getMaxAmount();

	/**
	 * Fill this resource with a certain amount
	 * 
	 * @param amount
	 *            The amount of this resource that is filled
	 * @return The amount of resources available after filling up
	 */
	public double fill(double amount);

	/**
	 * Try to extract a certain amount of this resource
	 * 
	 * @param amount
	 *            The amount of this resource that should be extracted
	 * @return The amount of this resource that was extracted
	 */
	public double extract(double amount);
}
