package de.tudarmstadt.maki.simonstrator.api.component.resourcedistribution;

import de.tudarmstadt.maki.simonstrator.api.common.UniqueID;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponent;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;


/**
 * A node that provides resources to {@link ResourceConsumer}s. Access to this interface is usually only performed
 * in simulations.
 *
 * @author Tim Feuerbach
 */
public interface ResourceDistributor extends HostComponent {
    /**
     * Returns a globally unique identifier for this distributor node.
     */
    public UniqueID getBeaconID();

    /**
     * Returns the distributor's current sequence ID.
     */
    public int getBeaconSequenceID();

    /**
     * Returns the distributor's location.
     */
    public Location getLocation();

    /**
	 * Returns a list of resources
	 */
	public DistributableResourceInventory getResources();

    /**
	 * Returns the number of resource units currently available at this distributor
	 * specified by the resource type.
	 */
	public double getResourceUnits(DistributableResource.RESOURCE_TYPE resourceType);

    /**
     * Returns whether the resource distributor has no more resources available.
     */
    public boolean isEmpty();

    /**
     * Reduces the number of resources at this distributor by the given value. If there are less resource units
     * available, the distributor will only be reduced by the amount of remaining units.
     *
     * @param demand how many resources to "take"
     * @return how many resources the consumer actually got (0 <= got <= demand)
     */
	public double takeResources(DistributableResource.RESOURCE_TYPE resourceType, double demand);

    /**
     * Adds a listener to this resource distributor.
     */
    public void registerListener(ResourceDistributorListener listener);
}
